/* buildgrid.c, Copyright (C) 2002, Tim McLarnan

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Do, please, let me hear from you if you use this package, or if you
have questions.

Tim McLarnan, Dept. of Mathematics, Earlham College, Richmond, IN 47374 USA
timm AT cs.earlham.edu

*/


/*
  buildgrid.c, Tim McLarnan, June, 2000.

  This program gets run as a cgi script, buildgrid.cgi, called from
  form1.php.  It takes the data collected from form1.php
  (the number of shafts and the source and target numbers of treadles)
  and builds the form containing the grid on which the user codes
  the original tie-up.

  It's silly to have written this in C, but I haven't yet taken the
  time to learn Perl or Python, and it was easier to use C
  than to learn a new language.  Since I had never used cgi before
  either, holding to a minimum the number of new technologies I was
  embracing simultaneously seemed prudent.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define INPUT_LENGTH 200


/*
  Gets the numerical value associated with <name> from the input
  string.  Since this will look like <name>=<value>, we scan for a
  number starting 1 character after the end of <name>.
  Returns -1 on various errors.
*/
int getNumericalValue(char *instring, char *name) {
  int value;
  char *start = strstr(instring, name);

  if (start == NULL) return(-1);
  start = start + strlen(name) + 1;
  if (start - instring >= INPUT_LENGTH-1) return(-1);
  if (sscanf(start, "%d", &value) == EOF) return(-1);
  return(value);
}
  

int main() {
  int row, column;
  char instring[INPUT_LENGTH];
  int ShaftCount; /* Number of shafts in the pattern. */
  int TargetCount; /* Number of treadles in the original pattern. */
  int TreadleCount; /* Number of treadles on user's loom. */
  const int
    MinShaft = 2,
    MaxShaft = 40,
    MinTarget = 2,
    MaxTarget = 100,
    MinTreadle = 2,
    MaxTreadle = 100;
  int winWidth;

  /* Write the header for the page we're building. */

  fgets(instring, INPUT_LENGTH, stdin);
  printf("Content-type: text/html\n\n");
  printf("<html><head><title>Original tie-up</title></head>\n");
  printf("<body bgcolor=\"#FFFFCC\" VLINK=#00AA00 ALINK=#C80000 ");
  printf("LINK=#C80000>\n");

  /* Now get the data from the user's form, and make sure it makes sense. */

  ShaftCount = getNumericalValue(instring, "ShaftCount");
  TargetCount = getNumericalValue(instring, "TargetCount");
  TreadleCount = getNumericalValue(instring, "TreadleCount");
  if ((ShaftCount < MinShaft) || (ShaftCount > MaxShaft)) {
    printf("<p>Sorry. The number of shafts must be between\n");
    printf("%d and %d.</p>\n", MinShaft, MaxShaft);
    printf("<p>Please go back to the previous page and try again.</p>\n");
    printf("<p>If your problems persist, please ");
    printf("<a href=\"http://www.cs.earlham.edu/~timm/mail.html\">");
    printf("send me mail.</a>\n");
    printf("</html>\n");
    exit(1);
  }
  if ((TargetCount < MinTarget) || (TargetCount > MaxTarget)) {
    printf("<p>Sorry. The number of treadles on the original tie-up must\n");
    printf("be between %d and %d.</p>\n", MinTarget, MaxTarget);
    printf("<p>Please go back to the previous page and try again.</p>\n");
    printf("<p>If your problems persist, please ");
    printf("<p>If your problems persist, please ");
    printf("<a href=\"http://www.cs.earlham.edu/~timm/mail.html\">");
    printf("send me mail.</a>\n");
    printf("</html>\n");
    exit(1);
  }
  if ((TreadleCount < MinTreadle) || (TreadleCount > MaxTreadle)) {
    printf("<p>Sorry. The number of treadles on your loom must be between\n");
    printf("%d and %d.</p>\n", MinTreadle, MaxTreadle);
    printf("<p>Please go back to the previous page and try again.</p>\n");
    printf("<p>If your problems persist, please ");
    printf("<p>If your problems persist, please ");
    printf("<a href=\"http://www.cs.earlham.edu/~timm/mail.html\">");
    printf("send me mail.</a>\n");
    printf("</html>\n");
    exit(1);
  }

  /* Now build the form with the grid of checkboxes. */

  winWidth = ( 25 * TargetCount > 500 ? 25 * TargetCount : 500);
  printf("<table border=\"1\" width=\"%d\" align=\"center\" ", winWidth);
  printf("bgcolor=\"#FFFFFF\" bordercolor=\"#000000\" cellpadding=\"5\">\n");
  printf("<tr><td valign=\"top\">\n");
  printf("<table border=\"0\" width=\"%d\" align=\"center\" ", winWidth - 10);
  printf("cellspacing=\"5\" cellpadding=\"5\">\n");
  printf("<tr><td valign=\"top\" height=\"20\">");
  printf("<font face=\"Arial, Helvetica, sans-serif\">");
  printf("<b><font size=\"-1\"><br></font></b></font></td>\n");
  printf("</tr><tr><td valign=\"top\" height=\"72\"><p><P>\n");
  printf("<center><h2>Tim's Rudimentary<br>Treadle Reducer</h2><br>\n");
  printf("</center>\n");
  printf("<form method=post action=\"weavereduce.cgi\">");
  printf("<input type=\"hidden\" name=\"ShaftCount\" value=\"%d\">\n",
	 ShaftCount);
  printf("<input type=\"hidden\" name=\"TargetCount\" value=\"%d\">\n",
	 TargetCount);
  printf("<table border cellspacing=\"5\" cellpadding=\"5\">\n");
  printf("<tr>\n");
  printf("<td>Number of shafts:</td>\n");
  printf("<td>%d</td>\n", ShaftCount);
  printf("</tr><tr>\n");
  printf("<td>Number of treadles in the original tie-up:</td>\n");
  printf("<td>%d</td>\n", TargetCount);
  printf("</tr><tr>\n");
  printf("<td>Number of treadles on your loom:</td>\n");
  printf("<td><input type=\"text\" name=\"TreadleCount\" size=\"4\" ");
  printf("value=\"%d\"></td>\n", TreadleCount);
  printf("</tr></table>\n");
  printf("<p>Click the check-boxes below to form the original tie-up.\n");
  printf("As usual in these diagrams, horizontal rows represent shafts,\n");
  printf("and vertical columns represent treadles.\n");
  printf("The diagram begins with no ties.  ");
  printf("Click buttons to designate ties.</p>\n");
  printf("<table border cellspacing=0 cellpadding=0>\n");
  printf("<tbody height=20 width=20 /tbody>\n");
  for (row = 0; row < ShaftCount; row++) {
    printf("<tr align=center>\n");
    for (column = 0; column < TargetCount; column++) {
      printf("<td><input type=\"checkbox\" name=\"Grid\" ");
      printf("value=\"%d %d\"></td>\n", row, column);
    }
    printf("</tr>\n");
  }
  printf("</table>\n");
  printf("<p>For many tie-ups, finding a reduction takes a second or two.\n");
  printf("On the other hand, for complicated tie-ups, and particularly in\n");
  printf("cases where no reduction to fewer treadles exists, the search\n");
  printf("can take minutes, hours, or, in theory, years.  (This is why\n");
  printf("you're using a computer instead of doing it by hand, right?)</p>\n");
  printf("As soon as a reduction is found, we'll quit and tell you.</p>\n");
  printf("<p>If we don't find a quick solution, what's the longest time\n");
  printf("you're willing to wait for an answer while we keep looking?\n</p>");
  printf("<p><b>I'm willing to give up after waiting</b></p>");
  printf("<table cellspacing=\"5\" cellpadding=\"5\">\n");
  printf("<tr>\n");
  printf("<td><input type=\"radio\" name=\"Timeout\" ");
  printf("value=\"10\"> 10 secs.</td>\n");
  printf("<td><input type=\"radio\" name=\"Timeout\" ");
  printf("value=\"30\" checked> 30 secs.</td>\n");
  printf("<td><input type=\"radio\" name=\"Timeout\" ");
  printf("value=\"60\"> 1 min.</td>\n");
  printf("<td><input type=\"radio\" name=\"Timeout\" ");
  printf("value=\"120\"> 2 mins.</td>\n");
  printf("</tr><tr>\n");
  printf("<td><input type=\"radio\" name=\"Timeout\" ");
  printf("value=\"600\"> 10 mins.</td>\n");
  printf("<td><input type=\"radio\" name=\"Timeout\" ");
  printf("value=\"1800\"> 30 mins.</td>\n");
  printf("<td><input type=\"radio\" name=\"Timeout\" ");
  printf("value=\"7200\"> 2 hrs.</td>\n");
  printf("<td><input type=\"radio\" name=\"Timeout\" ");
  printf("value=\"86400\"> 1 day.</td>\n");
  printf("</tr><tr>\n");
  printf("</tr></table>\n");
  printf("<input type=\"submit\" name=\"submit\" value=\"Find Reduction\">\n");
  printf("</form>\n");
  printf("<br><p>Tim McLarnan,<br>\n");
  printf("<br>Tremewan Professor of Mathematics\n");
  printf("<br><a href=\"http://www.earlham.edu\">Earlham College</a>,\n");
  printf("<br>Richmond, IN 47374 USA\n");
  printf("<br><br><a href=\"http://www.cs.earlham.edu/~timm/mail.html\">");
  printf("Send me mail.</a></p>\n");
  printf("</td></table></td></tr></table><p><BR></body></html>\n");
  exit(0);
}
